//#include "pub_def.h"

#ifndef _MC13192_REGS_H
#define _MC13192_REGS_H

// MC13192 soft reset 
#define RESET                   0x00

// Packet RAM 
#define RX_PKT                  0x01    // RX Packet RAM 
#define RX_PKT_LEN              0x2D    // RX Packet RAM Length [6:0] 
#define TX_PKT                  0x02    // TX Packet RAM 
#define TX_PKT_LEN              0x03    // TX Packet RAM Length 
#define TX_PKT_LEN_MASK         0x007F  // TX Packet RAM Length Mask 

// IRQ Status Register 
#define IRQ_MASK                0x05
#define STATUS_ADDR             0x24
#define RESIND_ADDR             0x25
#define TIMER1_IRQMASK_BIT      0x0001

// Mask and mode 
#define MODE_ADDR               0x06     
#define MODE2_ADDR              0x07     
#define LO1_COURSE_TUNE         0x8000

// Main Timer 
#define TIMER_PRESCALE          0x09

#define CURTIME_HI_ADDR					0x26
#define CURTIME_LO_ADDR					0x27
#define CURTIME_HI_MASK					0x00FF

#define TIMESTAMP_HI_ADDR       0x2e
#define TIMESTAMP_LO_ADDR       0x2f
#define TIMESTAMP_HI_MASK       0x00FF

// Frequency 
#define XTAL_ADJ_ADDR           0x0A
#define CLKS_ADDR               0x0A
#define LO1_IDIV_ADDR           0x0F
#define LO1_NUM_ADDR            0x10
#define PRESCALE_ADDR           0x09

// Timer comparators 
#define T1_HI_ADDR              0x1B    
#define T1_LO_ADDR              0x1C
#define T2_HI_ADDR              0x1D    
#define T2_LO_ADDR              0x1E

// CCA 
#define CCA_THRESHOLD           0x04
#define CCA_RESULT_ADDR         0x2D
#define FEGAIN_ADDR             0x04

// TX 
#define PA_ADJUST_ADDR          0x12

// GPIO 
#define GPIO_CONFIG             0x0B
#define GPIO_DATA               0x0C
#define GPIO_DATA_MASK          0x003F

// Version 
#define VERSION_REG             0x2C
#define VERSION_MASK            0x1C00

//	Test registers
#define BER_REG                 0x30
#define BER_MASK                0x8000
#define PSM_REG                 0x31
#define PSM_MASK                0x0008
#define PAEN_REG                0x08
#define PAEN_MASK               0x8000

#endif // _MC13192_REGS_H 

//	Modes defined for the transciever
#define IDLE_MODE								0x00
#define RX_MODE									0x02
#define TX_MODE									0x03
#define PULSE_TX_MODE						0x43
#define CCA_MODE								0x01
#define RX_MODE_WTO							0x80
#define HIBERNATE_MODE					0x81
#define DOZE_MODE								0x82
#define IDLE_MODE_DOZE					0x83
#define IDLE_MODE_ATTN					0x84
#define TIMEOUT									0x85
#define SYSTEM_RESET_MODE				0x86    /* Not implemented */
#define MC13192_RESET_MODE			0x87    /* Not implemented */
#define MC13192_CONFIG_MODE			0x88    /* Not implemented */
#define RESET_DELAY							0x89    /* Set to 100ms    */

/* Test modes defined for the transciever */
#define PULSE_TX_PRBS9					0x01
#define FORCE_IDLE							0x02
#define CONTINUOUS_RX						0x03
#define CONTINUOUS_TX_MOD				0x04
#define CONTINUOUS_TX_NOMOD			0x05

#define MAXPACKETSIZE 		126

#define MAX_POWER 100	/* Numbers chosen arbitrarily but > 16 */
#define NOMINAL_POWER 0x0B
#define MIN_POWER 50	/* Numbers chosen arbitrarily but > 16 */

// Status enumations for the PHY
#define SUCCESS		0x77
#define	CRCERROR	0x5

#define INITIAL_VALUE 0x0

//	Status of DataMode
#define	SET_IDLE		0
#define	SET_TXDATA	1
#define	SET_RXDATA	2

#define	countof(x)	(sizeof(x)/(sizeof((x)[0])))

//	Modes defined for the transciever
#define IDLE_MODE								0x00
#define RX_MODE									0x02
#define TX_MODE									0x03
#define PULSE_TX_MODE						0x43
#define CCA_MODE								0x01
#define RX_MODE_WTO							0x80
#define HIBERNATE_MODE					0x81
#define DOZE_MODE								0x82
#define IDLE_MODE_DOZE					0x83
#define IDLE_MODE_ATTN					0x84
#define TIMEOUT									0x85
#define	SYSTEM_RESET_MODE				0x86
#define MODEM_RESET_MODE				0x87    // Not implemented
#define MODEM_CONFIG_MODE				0x88
#define RESET_DELAY							0x89    // Set to 100ms

// Test modes defined for the transciever
#define PULSE_TX_PRBS9					0x01
#define FORCE_IDLE							0x02
#define CONTINUOUS_RX						0x03
#define CONTINUOUS_TX_MOD				0x04
#define CONTINUOUS_TX_NOMOD			0x05

#define TRUE  1
#define FALSE 0

#define MAXPACKETSIZE 		126

#define MAX_POWER 100	// Numbers chosen arbitrarily but > 16
#define NOMINAL_POWER 0x0B
#define MIN_POWER 50	// Numbers chosen arbitrarily but > 16

// Status enumations for the PHY
#define SUCCESS		0x77
#define	CRCERROR	0x5
#define OVERFLOW	1

#define INITIAL_VALUE 0x0

//	Status of DataMode
#define	SET_IDLE		0
#define	SET_TXDATA	1
#define	SET_RXDATA	2


// Status enumations for the PHY.
/*
enum pd_data_status {RX_ON = 1, TRX_OFF};
enum mc13192_power_modes {RF_POWER_ON = 1, RF_POWER_HIBERNATE, RF_POWER_DOZE};
enum PLME_set_trx_state_request {ERROR = 1};
*/