//
//	Includes
//
#include	"MOTTYPES.h"
#include	"Struct.h"
#include	"PortDef.h"
#include	<string.h>
//
//	Prototypes
//
#include	"ProtoType.h"

//	initialisation constants
#define baud57600 0x09  // 8.66
#define	baud38400	0x0D	// Value = 500000 / baudrate
#define	baud19200	0x1a	// 19200bps
#define	baud9600	0x34	// 9600bps

//	Variables
UINT8	RxRingBuf[1024];	// Ring buffer
volatile UINT16	RxRingOut, RxRingIn;	// Ring pointer
#define	RX_MASK	0x3ff
#define	RX_FULL		0x300
#define	RX_EMPTY	0x100

UINT8	TxRingBuf[1024];	// Ring buffer
volatile UINT16	TxRingOut, TxRingIn;	// Ring pointer
#define	TX_MASK	0x3ff

extern tJumpTable HookTable[];

//
//	SCI RX interrupt handler
//
interrupt void SciRxInterrupt()
{	
UINT8	SciStat, SciData;
	SciStat = SCIS1;	// Get SCI status register
	if(!(SciStat & 0x20))		// Noise or double int?
		return;					// Do nothing
	SciData = SCID;	// Read data from SCI
//	It's good data then store to buffer
	if(RxRingOut != ((RxRingIn + 1) & RX_MASK))	// Have space to put?
	{
		RxRingBuf[RxRingIn] = SciData;	// Set RX data to ring buffer
		RxRingIn = ((RxRingIn + 1) & RX_MASK);// Advance ring pointer
	}
//	Flow control (Stop)
	if(RTS_DATA == RTS_ON	// Now flow is on?
	 && (((RxRingIn - RxRingOut) & RX_MASK) > RX_FULL))// No space to put?
	{
		RTS_DATA = RTS_OFF;	// Set RTS off
		LED3_OFF;						// Show flow is off
	}
}

//
//	SCI TX interrupt handler
//
interrupt void SciTxInterrupt()
{	
UINT8	SciStat;
	SciStat = SCIS1;
	if(!(SciStat & 0x80))	// Tx buffer is empty?
		return;							// no then do nothing
	if(TxRingIn == TxRingOut)	// Tx buffer ends?
	{
		SCIC2_TIE = 0;			// Disable transmit 
		return;
	}
	SCID = TxRingBuf[TxRingOut];	// Send one char
	TxRingOut = ((TxRingOut + 1) & TX_MASK);	// Advance ring index
}

//
//	SCI Send data by timer
//
void SciTxStart()
{
	SCIC2_TIE = 1;				// Start TX transmission
}

//
//	SCI Error interrupt handler
//
interrupt void SciErrInterrupt()
{	
UINT8	SciStat;
	SciStat = SCIS1; // Read status register
	SciStat = SCID;	// Clear error status
}


//
//	Check Sci data has come?
//
UINT8 SciCheck(void)
{
	return (RxRingIn != RxRingOut);	// Check ring pointer difference
}

//
//	Sense key data
//
UINT8 SenseKey(void)
{
UINT8	getc;
	if(!SciCheck())
		return 0;
	getc = RxRingBuf[RxRingOut];
	RxRingOut = (RxRingOut + 1) & RX_MASK;
	return getc;
}

//
//	Get one char from SCI
//
UINT8 SciGetc(void)
{
UINT8	getc;
	while(!SciCheck());
	getc = RxRingBuf[RxRingOut];
	RxRingOut = (RxRingOut + 1) & RX_MASK;
	return getc;
}

//
//	Put one char to SCI
//
void SciPutc(UINT8 c)
{
	while(!SCIS1_TDRE); // Check TX data buffer empty
	while(!SCIS1_TC);	 // wait for Tx complete
	SCID = c;	 // load data to SCI2 register
}

//
//	Put CR to SCI
//
void SciCrlf(void)
{
	SciPutc('\r');
	SciPutc('\n');
}

void SciPuts(UINT8 *String)
{
	while(*String)
	{
		if( *String == '\n' )
			SciCrlf();
		else
			SciPutc(*String);
		String++;
	}
}

//
//	Get one line from SCI
//
void SciGets(UINT8 *Buffer, UINT16 Length, UINT8 echo)
{
UINT8	c, Ptr;

	if(Length == 0) return;	// Called with zero length
	Ptr = 0;
	Length--;	// Reserve space for '\0'
	for(;;)
	{
		c = SciGetc();
		if(c == '\xd') break;		// CR
		if(c == '\x8' && Ptr)		// BS
		{
			if(echo)
			{
				SciPutc(c);
				SciPutc(' ');
				SciPutc(c);
			}
			Ptr--;
		}
		else if(c >= ' ' && Ptr < Length)
		{
			Buffer[Ptr] = c;
			if(echo)
				SciPutc(c);
			Ptr++;
		}
	}
	Buffer[Ptr] = '\0';
	if(echo)
		SciCrlf();
}
//
//	Hex print for 4 bit
//
void PutHex1(UINT8 val)
{
	val = val & 0xf;
	if(val < 10)
		SciPutc(val + 0x30);
	else
		SciPutc(val + 0x57);
}
//
//	Hex print for byte
//
void PutHex2(UINT8 val)
{
	PutHex1(val >> 4);
	PutHex1(val);
}
//
//	Hex print for short
//
void PutHex4(UINT16 val)
{
	PutHex2(val >> 8);
	PutHex2((UINT8 )val);
}
//
//	Dec print for byte
//
void PutDec2(UINT8 val)
{
	if(val >= 10)
	{
		SciPutc('0' + (val / 10));
		val = (UINT8)(val % 10);
	}
	else
		SciPutc(' ');
	SciPutc('0' + val);
}

//
//
//
void RxEnable(void)
{
	SCIC2_RIE = 1;	// Enable RX interrupts
	RTS_DATA = RTS_ON;	//RTS_OFF;		//	Stop flow for begining
	LED3_ON;
}

//
//
//
void RxDisable(void)
{
	SCIC2_RIE = 0;	// Disable RX interrupts
	RTS_DATA = RTS_OFF;	//RTS_OFF;		//	Stop flow for begining
	LED3_OFF;
}

//
//	Init SCI registers
//
void	SciInit(void)
{
	SCIBDH = 0x00;		// Set baudrate high byte
	SCIBDL = 11;			// Set baudrate 56818 at 20MHz clock

	SCIC2 = 0b10001100;	// TX interrupt & TX & RX enable
	SCIC3 = 0b00001110;	// Enable OR, NE, FE error interrupt
	RxRingOut = RxRingIn = 0;	// Clear ring buffer pointer
	TxRingOut = TxRingIn = 0;	// Clear ring buffer pointer
#ifdef CAM
	HookTable[18].adrs = SciErrInterrupt;	// Set Error interrupts
	HookTable[19].adrs = SciRxInterrupt;	// Set Rx function address
	HookTable[20].adrs = SciTxInterrupt;	// Set Tx ISR address
#else
	HookTable[15].adrs = SciErrInterrupt;	// Set Error interrupts
	HookTable[16].adrs = SciRxInterrupt;	// Set Rx function address
	HookTable[17].adrs = SciTxInterrupt;	// Set Tx ISR address
#endif
}
